/*
 * IofeCompletion.c: IoFE completion code.
 * Copyright (c) 2004 Kentaro A. Kurahone.  All Rights Reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <histedit.h>

#include "IoState.h"
#include "IoObject.h"
#include "IoSeq.h"

#include "IofeCompletion.h"

/* External globals. */
extern int isMulti;

/* Static globals. */
static IoState *iovmState = NULL;
static const char ioCompCommandPrefix[] = "IoFE Completion doComplete(\"";
static const char ioCompCommandSuffix[] = "\")";

void initCompletion(IoState *state, IoObject *where)
{
  iovmState = state;

  /* Create completion object. */
  IoCompletionInit(state, where);
}

unsigned char invokeCompletion(EditLine *e, int ch)
{
  IoObject *result;
  IoSeq *str;
  const LineInfo *lineInfo;
  char *currentLine;
  char *ioCmd;

  lineInfo = el_line(e);

  /* Only complete if the cursor is at the end of the buffer,
   * and we're not in multi line input land.
   */
  if (lineInfo->cursor == lineInfo->lastchar && !isMulti) {
    /* Extract current line, and stash in a buffer. */
    currentLine = malloc(lineInfo->lastchar - lineInfo->buffer + 1);
    if (currentLine == NULL) {
      /* Out of memory. */
      fprintf(stderr, "Out of memory. Aborting.\n");
      exit(-1);
    }
    strncpy(currentLine,
      lineInfo->buffer,
      lineInfo->lastchar - lineInfo->buffer);
    currentLine[lineInfo->lastchar - lineInfo->buffer] = '\0';

    ioCmd = (char *) malloc(strlen(ioCompCommandPrefix)
      + strlen(ioCompCommandSuffix)
      + strlen(currentLine)
      + 1);
    sprintf(ioCmd, "%s%s%s",
      ioCompCommandPrefix,
      currentLine,
      ioCompCommandSuffix);

    result = IoState_doCString_(iovmState, ioCmd);
    if (result == iovmState->ioNil) {
      fflush(stdout);
    } else {
      str = (IoSeq *) result;
      el_insertstr(e, CSTRING(str));
    }

    free(ioCmd);
    free(currentLine);

    return CC_REDISPLAY; /* Redisplay to take completion into account. */
  }

  return CC_ERROR;
}
